﻿using System;
using System.Collections.Generic;
using Ozeki.Media;
using Ozeki.Network;
using Ozeki.VoIP;

namespace _07_Parallel_calls
{
    class Program
    {
        static List<CallHandler> _callHandlers;

        static void Main(string[] args)
        {
            _callHandlers = new List<CallHandler>();
            var softphone = new Softphone();

            ShowHelp();

            SIPAccountInitialization(softphone);

            softphone.IncomigCall += softphone_IncomigCall;
            Console.ReadLine();
        }

        private static void ShowHelp()
        {
            Console.WriteLine("\nThis is an example code for managing simultaneous calls written in C#.");
            Console.WriteLine("\n - You will need to setup your SIP account (your authentication ID, username, display name etc.).");
            Console.WriteLine("\n - Afterwards, the program will be able to receive multiple simultaneous calls automatically. The callers will listen to a sample mp3 song.\n");
            Console.WriteLine("-------------------------------------------------------------------------------\n");
        }

        static void softphone_IncomigCall(object sender, VoIPEventArgs<Ozeki.VoIP.IPhoneCall> e)
        {
            Console.WriteLine("Incoming call!");
            var callHandler = new CallHandler(e.Item);
            callHandler.Completed += callHandler_Completed;

            lock (_callHandlers)
                _callHandlers.Add(callHandler);

            callHandler.Start();
        }


        static void callHandler_Completed(object sender, EventArgs e)
        {
            lock (_callHandlers)
                _callHandlers.Remove((CallHandler)sender);
        }

        private static void SIPAccountInitialization(Softphone softphone)
        {
            Console.WriteLine("Please setup your SIP account!\n");
            Console.WriteLine("Please set your authentication ID: ");
            var authenticationId = Read("authenticationId", true);

            Console.WriteLine("Please set your user name (default:" +authenticationId +"): ");
            var userName = Read("userName", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            Console.WriteLine("Please set your name to be displayed (default: " +authenticationId +"): ");
            var displayName = Read("displayName", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            Console.WriteLine("Please set your registration password: ");
            var registrationPassword = Read("registrationPassword", true);

            Console.WriteLine("Please set the domain name (default: your local host): ");
            var domainHost = Read("domainHost", false);
            if (string.IsNullOrEmpty(domainHost))
                domainHost = NetworkAddressHelper.GetLocalIP().ToString();

            Console.WriteLine("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("domainPort", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register...\n");
            softphone.Register(true, displayName, userName, authenticationId, registrationPassword, domainHost, domainPort);
        }

        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                {
                    return input;
                }

                if (!string.IsNullOrEmpty(input))
                {
                    return input;
                }

                Console.WriteLine(inputName +" cannot be empty!");
                Console.WriteLine(inputName +": ");
            }
        }
    }
}
